module HT =
struct

  let hash a = Hashtbl.hash a (* fonction générique d'OCaml *)
  let equal a b = a = b

  type ('key, 'value) t =
    {
      mutable data : ('key * 'value) list array;
      mutable size : int; (* nombre d'éléments dans la table *)
    }

  let create () = { data = Array.make 4 [];
                    size = 0}


  let find t k =
    let idx = (hash k) mod (Array.length t.data) in
    let buckets = t.data.(idx) in
    List.assoc k buckets


  let simple_add t k v =
    let idx = (hash k) mod (Array.length t.data) in
    let buckets = t.data.(idx) in
    let rec loop l =
      match l with
        [] -> t.size <- t.size + 1; [(k, v)]
      | (k0, v0) :: ll ->
        if equal k0  k then (k0, v) :: ll
        else (k0, v0) :: loop ll
    in
    t.data.(idx) <- loop buckets


  let iter f t =
    for i = 0 to Array.length t.data - 1 do
      List.iter (fun (k, v) -> f k v) t.data.(i);
    done

  let resize t =
    let nt = { data = Array.make (2 * Array.length t.data) [];
               size = 0; }
    in
    iter (simple_add nt) t;
    t.data <- nt.data;
    t.size <- nt.size

  let add t k v =
    simple_add t k v;
    while t.size > 2 * Array.length t.data do
      resize t;
    done


  let debug t pk pv =
    for i = 0 to Array.length t.data - 1 do
      let l = List.map (fun (k, v) ->
        Printf.sprintf "(%s, %d, %s)"
        (pk k) (Hashtbl.hash k) (pv v)) t.data.(i)
      in
      let s = "[" ^ String.concat "; " l ^ "]" in
      Printf.printf "% 3d -> %s\n" i s
    done

end

let ht = HT.create ()
let clear_screen () =
  Printf.printf "\x1b[2J\x1b[H%!"
let () = for i = 0 to 32 do
  HT.add ht i "A";
  clear_screen ();
  HT.debug ht string_of_int (fun x -> x);
  flush stdout;
  Unix.sleep 1;
done